#include <cstddef>
using std::size_t;

#include <iostream>
using std::cout;
using std::endl;

#include <iomanip>
using std::setw;

#include <cmath>
using std::log;
using std::exp;

#include <algorithm>
using std::min;

#include "bitmap.hpp"

int main()
{
	const size_t M = 45*15;
	const size_t N = 30*15;
	unsigned int escape[M][N];
	
	const unsigned int nmax = 500;
	const double rmax_sqr = 9.0;
	double geo_ave = 0.0;
	unsigned int ave_count = 0;
	
	for (size_t i = 0; i < M; ++i)
		for (size_t j = 0; j < N; ++j)
		{
			const double real_c = 3.0*i/M-2.0;
			const double imag_c = 2.0*j/N-1.0;
			unsigned int n = 0;
			double real_z = real_c;
			double imag_z = imag_c;
			while (n < nmax and real_z*real_z + imag_z*imag_z < rmax_sqr)
			{
				const double real_z_updated = real_z*real_z-imag_z*imag_z + real_c;
				const double imag_z_updated = 2.0*real_z*imag_z + imag_c;
				real_z = real_z_updated;
				imag_z = imag_z_updated;
				++n;
			}
			escape[i][j] = n;
			if (n != nmax)
			{
				geo_ave += log(double(n));
				++ave_count;
			}
		}

	geo_ave = exp(geo_ave/ave_count);

	bitmap image(M, N, 1.0);
	
	for (size_t j = 0; j < N; ++j)
		for (size_t i = 0; i < M; ++i)
		{
			const double c = (escape[i][j] == nmax ? 1.0 : min(1.0,0.2*escape[i][j]/geo_ave));
			image.set_pixel(i,j,1-c,1-c*c,1-c*c*c);
		}

	image.write("out.png");
	
	return 0;
}

